"""
Copyright 2021 Xilinx Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

"""
MIT License

Copyright (c) 2016 Gabriel Stanovsky

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

""" Usage:
    tabReader --in=INPUT_FILE

Read a tab-formatted file.
Each line consists of:
sent, prob, pred, arg1, arg2, ...

"""

from oie_readers.oieReader import OieReader
from oie_readers.extraction import Extraction
from docopt import docopt
import logging

logging.basicConfig(level = logging.DEBUG)

class TabReader(OieReader):

    def __init__(self):
        self.name = 'TabReader'

    def read(self, fn):
        """
        Read a tabbed format line
        Each line consists of:
        sent, prob, pred, arg1, arg2, ...
        """
        d = {}
        ex_index = 0
        with open(fn) as fin:
            for line in fin:
                if not line.strip():
                    continue
                data = line.strip().split('\t')
                text, confidence, rel = data[:3]
                curExtraction = Extraction(pred = rel,
                                           sent = text,
                                           confidence = float(confidence))
                ex_index += 1

                for arg in data[3:]:
                    curExtraction.addArg(arg)

                d[text] = d.get(text, []) + [curExtraction]
        self.oie = d


if __name__ == "__main__":
    args = docopt(__doc__)
    input_fn = args["--in"]
    tr = TabReader()
    tr.read(input_fn)




# Copyright 2019 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collector
import platform
import vaitraceSetting
import os
import logging

FTRACE_DIR = "/sys/kernel/debug/tracing/"


class Ftrace:
    def __init__(self, _name, _path=FTRACE_DIR, _buffersize=8192, _type=None, _clk='local', saveTo=None, _debug=False):
        self.cwd = os.getcwd()
        self.instPath = ""
        self.debug = _debug
        self.path = _path
        self.traceHandler = None
        self.saveTo = saveTo
        self.name = _name
        self.traceClock = _clk
        self.traceType = _type
        self.quirk = False

        if vaitraceSetting.checkFtrace() == False:
            logging.error("Ftrace not Enable")

        if self.checkKernelRelease()[0] < 4:
            self.legacyKernel = True
        else:
            self.legacyKernel = False
        if (self.legacyKernel) and (self.traceType == 'uprobe'):
            self.quirk = True

        self.buffersize = _buffersize
        self.createInstance(_name, _path)
        self.setBufferSize(_buffersize)
        self.setClockSource()

    def __del__(self):
        """Delete ftrace instance"""
        os.system("rmdir %s" % self.instPath)

        self.disableAllEvents()
        self.clearUprobe()

    def backCwd(self):
        if self.cwd == "":
            raise IOError()
        os.chdir(self.cwd)

    def createInstance(self, _name, path):
        # if self.legacyKernel == True and self.traceType == 'user':
        if False:
            """
            For Linux Kernel-3.x, do not use [instance],
            just create trace points in /sys/kernel/debug/tracing dir
            """
            self.name = _name
            os.chdir(path)
            self.instPath = os.getcwd()
            self.backCwd()
        else:
            # if self.legacyKernel == False:
            try:
                """Create ftrace instance"""
                os.chdir(os.path.join(path, "instances"))
                os.mkdir(_name)
            except IOError:
                if os.path.exists(_name) and os.path.isdir(_name):
                    pass
                else:
                    raise IOError("Create instance failed")
            finally:
                os.chdir(_name)
                self.name = _name
                self.instPath = os.getcwd()
                self.backCwd()

    def checkKernelRelease(self):
        kRelease = platform.uname().release.split('.')
        return (int(kRelease[0]), int(kRelease[1]))

    def _toTraceInstDir(func):
        def wrapper(self, *args, **kw):
            backInst = (os.getcwd() == self.instPath)

            os.chdir(self.instPath)
            ret = func(self, *args, **kw)
            if not backInst:
                self.backCwd()
            return ret
        return wrapper

    def _toTraceDir(func):
        def wrapper(self, *args, **kw):
            os.chdir(self.path)
            ret = func(self, *args, **kw)
            self.backCwd()
            return ret
        return wrapper

    @_toTraceInstDir
    def setBufferSize(self, bufsize):
        if self.quirk:
            os.chdir(self.path)
        open("buffer_size_kb", 'w').write(str(bufsize))

    @_toTraceInstDir
    def setClockSource(self):
        traceClock = self.traceClock
        open("trace_clock", 'w').write(traceClock)

    @_toTraceInstDir
    def enableTracing(self):
        if self.quirk:
            os.chdir(self.path)
        logging.debug("### enable tracing %s" % os.getcwd())
        open("tracing_on", 'w').write("1")

    @_toTraceInstDir
    def disableTracing(self):
        if self.quirk:
            os.chdir(self.path)
        logging.debug("### disable tracing %s" % os.getcwd())
        open("tracing_on", 'w').write("0")

    @_toTraceInstDir
    def enableEvent(self, system, event, filter=""):
        """
        For example, system might be [sched] and event [sched_switch]
        Then we should write 1 to ./events/[sched]/[sched_switch]/enable
        echo '(prev_comm == test_dnndk_resn) || (next_comm == test_dnndk_resn)' > ./events/sched/sched_switch/filter
        echo 1 > ./events/sched/sched_switch/enable
        """
        if (filter != ""):
            path = os.path.join("./", "events", system, event, "filter")
            open(path, 'w').write(filter)

        path = os.path.join("./", "events", system, event, "enable")
        if os.path.exists(path) == True:
            open(path, 'w').write("1")

    @_toTraceInstDir
    def disableEvent(self, system, event):
        """
        For example, system might be [sched] and event [sched_switch]
        Then we should write 0 to ./events/[sched]/[sched_switch]/enable
        echo 0 > ./events/sched/sched_switch/enable
        """
        path = os.path.join("./", "events", system, event, "enable")
        if os.path.exists(path) == True:
            open(path, 'w').write("0")

    @_toTraceDir
    def addUprobe(self, func, lib, offset):
        """
        This func will add uprobe and uretprobe event at the same time
        lib should be abspath
        example:
        echo 'p:runResnet50_entry /root/lttng/resnet50/test_dnndk_resnet50:0x448c' >> /t/uprobe_events
        echo 'r:runResnet50_exit /root/lttng/resnet50/test_dnndk_resnet50:0x448c' >> /t/uprobe_events
        """

        os.chdir(self.path)
        """uprobe_event can accept symbol length <= 64 characters"""
        limit = 64 - len("_entry")
        if len(func) >= limit:
            logging.debug("Too long symbol name [%s]" % (func))
            func = func[::-1][:limit:][::-1]

        cmdU = "p:" + func + "_entry " + lib + ":" + offset
        cmdUret = "r:" + func + "_exit " + lib + ":" + offset

        logging.debug("### uprobe %s" % cmdU)
        if self.debug:
            print("### uprobe %s" % cmdU)
            print("### uretprobe %s" % cmdUret)

        # open failed ?????
        #open("./uprobe_events", "a+").writeline(cmdU)
        #open("./uprobe_events", "a+").writeline(cmdUret)

        ret = os.system("echo %s >> ./uprobe_events" % cmdU)
        if ret != 0:
            logging.warning("[%s] event un-match" % cmdU)
            return
        ret = os.system("echo %s >> ./uprobe_events" % cmdUret)
        if ret != 0:
            logging.warning("[%s] event un-match" % cmdUret)

        self.enableUprobe(func)
        #print(open("./uprobe_events", "r").read())

    @_toTraceDir
    def addKprobe(self, name, mod, offset, fetchargs: []):
        """
        This func will add kprobe event
        lib should be abspath
        example:
        echo 'p:cu_start zocl:zocl_hls_start cu_idx=+0(%x0)' >> kprobe_events
        echo 'p:cu_done zocl:zocl_hls_check+0x64 cu_idx=+0(%x21)' >> kprobe_events
        """

        os.chdir(self.path)
        cmd = "p:%s %s:%s" % (name, mod, offset)
        for arg in fetchargs:
            cmd += " %s" % arg

        if self.debug:
            logging.debug("### kprobe %s" % cmd)

        os.system("""echo '%s' >> ./kprobe_events""" % cmd)

        self.enableKprobe(name)

    @_toTraceDir
    def clearUprobe(self):
        if self.debug:
            logging.debug("Clear all Uprobe & Uretprobe events")

        # open failed ?????
        #open("./uprobe_events", "a+").writeline(cmdU)
        #open("./uprobe_events", "a+").writeline(cmdUret)

        #print("echo  > ./uprobe_events", self.instPath)
        os.system("echo  > ./uprobe_events")

    @_toTraceDir
    def disableAllEvents(self):
        if self.debug:
            print("Disable all events")

        # open failed ?????
        #open("./uprobe_events", "a+").writeline(cmdU)
        #open("./uprobe_events", "a+").writeline(cmdUret)

        os.system("echo 0 > ./events/enable")

    @_toTraceInstDir
    def enableUprobe(self, func):
        self.enableEvent("uprobes", func+"_entry")
        self.enableEvent("uprobes", func+"_exit")

    @_toTraceInstDir
    def enableKprobe(self, probePoint):
        self.enableEvent("kprobes", probePoint)

    @_toTraceInstDir
    def disableUprobe(self, func):
        self.disableEvent("uprobes", func+"_entry")
        self.disableEvent("uprobes", func+"_exit")

    @_toTraceInstDir
    def getTrace(self):
        # if self.legacyKernel == True and self.traceType == 'uprobe':
        if self.quirk:
            os.chdir(self.path)

        trace = open("trace", "r").readlines()

        if self.saveTo != None:
            f = open(self.saveTo, "wt+")
            logging.debug("Trace file was saved to [%s]" % self.saveTo)
            f.writelines(trace)
            f.close()

        return trace

    @_toTraceInstDir
    def startTracePipe(self):
        self.traceHandler = Popen(
            ["cat", "trace_pipe"], stdout=PIPE, stderr=PIPE)

    @_toTraceInstDir
    def stopTracePipe(self, saveTo=""):
        t = self.traceHandler
        t.terminate()

        trace = t.stdout.readlines()

        if saveTo != "":
            f = gzip.open(saveTo, "wt")
            f.writelines(trace)
            f.close()

        return trace

    @_toTraceInstDir
    def clearTracing(self, disable=True):
        if disable:
            self.disableTracing()
        open("trace", 'w').write('')


class ftraceCollector(collector.collectorBase.Collector):
    def __init__(self):
        super().__init__(name='ftrace')
        self.instances = []

    def __del__(self):
        pass

    def prepare(self, conf: dict) -> dict:
        """
        type: "kprobe" | "uprobe" | "event"
        list for kprobe: [name, mod, offset, fetchargs: []]
        list for uprobe: [func, lib, offset]
        list for event : [system, event, filter=""]

        """

        ftraceOption = conf.get('collector', {}).get('ftrace')
        if ftraceOption == None:
            return conf

        traceClock = conf.get('control', {}).get('traceClock', 'global')
        open(os.path.join(FTRACE_DIR, "trace_clock"), 'w').write(traceClock)

        for name in ftraceOption.keys():
            instOption = ftraceOption[name]
            traceType = instOption.get('type')
            if instOption.get('name', None) != name:
                assert()

            _saveTo = instOption.get('saveTo', None)

            if _saveTo != None:
                _saveTo = os.path.abspath(_saveTo)
            traceInst = Ftrace(name, saveTo=_saveTo,
                               _clk=traceClock, _type=traceType)

            if traceType == 'kprobe':
                # dpu.addKprobe("cu_start", "zocl", "zocl_hls_start", ["cu_idx=+0(%x0):u32"])
                for t in instOption.get("traceList", None):
                    traceInst.addKprobe(*t)
            elif traceType == 'uprobe':
                # addUprobe(func.name, func.libPath, "0x%x" % func.offset)
                for t in instOption.get("traceList", None):
                    traceInst.addUprobe(*t)
            elif traceType == 'event':
                for t in instOption.get("traceList", None):
                    traceInst.enableEvent(*t)
            else:
                assert()

            self.instances.append(traceInst)

        for ft in self.instances:
            ft.clearTracing()

        return conf

    def start(self):
        super().start()
        logging.debug("### ftrace start")

        for inst in self.instances:
            inst.enableTracing()

    def stop(self):
        super().stop()

        logging.debug("### ftrace stop")

        for inst in self.instances:
            inst.disableTracing()

    """
    data = {
        ftrace: {
            cuEdge: xxxxxx,
            sched: xxxxxx,
        }
    }
    
    """

    def getData(self):
        ftraceData = {}

        for inst in self.instances:
            ftraceData.update({inst.name: inst.getTrace()})

        return ftraceData


collector.collectorBase.register(ftraceCollector())



#!/usr/bin/python3
# -*- coding:utf-8 -*-

# Copyright 2019 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from ctypes import *
import logging

import tracer.tracerBase
import copy
import os


class APMRecord(Structure):
    _fields_ = [('time', c_double), ('data', c_uint * 10)]

    def output(self):
        return "APM %.7f %d %d %d %d %d %d %d %d %d %d\n" % (self.time, *self.data)


class APM:
    def __init__(self, path='/usr/lib/libxapm.so', enable=True):
        self.apmLib = cdll.LoadLibrary(path)
        self.data = []
        self.interval = 0
        self.enabled = enable

    def start(self, interval=1.0):
        if self.enabled == False:
            return
        self.interval = interval
        self.apmLib.apm_start.restype = c_double
        return self.apmLib.apm_start(c_double(interval))

    def pushData(self, data):
        if self.enabled == False:
            return
        for i in range(0, len(data.data)):
            data.data[i] = data.data[i]

        self.data.append(copy.deepcopy(data))

    def printData(self, data):
        if self.enabled == False:
            return
        info = str()
        info += "TimeStamp: %.7f\n" % data.time

        read = [data.data[i]/self.interval/1024 /
                1024 for i in range(0, 10) if i % 2 == 0]
        info += "Read Ports:  "
        for d in read:
            info += "%8.1f" % d

        info += " MB/s\n"

        write = [data.data[i]/self.interval/1024 /
                 1024 for i in range(0, 10) if i % 2 == 1]
        info += "Write Ports: "
        for d in write:
            info += "%8.1f" % d

        info += " MB/s\n  "

        return info

    def stop(self):
        if self.enabled == False:
            return
        self.apmLib.apm_stop()

        data = APMRecord()
        pd = pointer(data)

        while (self.apmLib.apm_pop_data(pd) == 0):
            self.data.append(copy.deepcopy(data))

    def transTimebase(self):
        for i in range(0, len(self.data)):
            for j in range(0, len(self.data[i].data)):
                self.data[i].data[j] = int(
                    self.data[i].data[j] / self.interval)


def checkAPM():
    if os.path.exists("/dev/uio1") == False:
        return False

    """/sys/class/uio/uio1/device/of_node/name"""
    if os.path.exists("/sys/class/uio/uio1/device/of_node/name") == False:
        return False

    with open("/sys/class/uio/uio1/device/of_node/name", "rt") as f:
        try:
            name = f.read().strip()
            if name.startswith('perf-monitor') == False:
                return False
        except:
            return False

    return True


class xapmTracer(tracer.tracerBase.Tracer):
    def __init__(self):
        super().__init__('xapm', source=[],
                         compatible={'machine': ["aarch64"]})
        self.apm = None

    def prepare(self, option: dict, debug: bool):
        "Handle Input Options"
        xapmOption = option.get('tracer', {}).get('xapm', {})
        self.interval = xapmOption.get("APM_interval", 0.01)
        self.apm = APM()

        "Handle Output Options"
        return option

    def start(self):
        super().start()
        # self.timesync = self.apm.start(self.interval)
        self.apm.start(self.interval)

    def stop(self):
        super().stop()
        self.apm.stop()

    def process(self, data, t_range=[]):
        self.apm.transTimebase()

    def compatible(self, platform: {}):
        if super().compatible(platform) == False:
            return False

        return checkAPM()

    def getData(self):
        return [d.output() for d in self.apm.data]


if __name__ == '__main__':
    apm = APM()
    apm.start(0.001)
    time.sleep(1)
    apm.stop()

    for a in apm.data:
        print(apm.printData(a))

else:
    tracer.tracerBase.register(xapmTracer())



# Copyright 2019 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from writer.parser.timelineUtil import *
import re

TraceEventType = [
    "VAI_EVENT_HOST_START",
    "VAI_EVENT_HOST_END",
    "VAI_EVENT_INFO",

    "VAI_EVENT_DEVICE_START",
    "VAI_EVENT_DEVICE_END",
    "VAI_EVENT_MARKER",
    "VAI_EVENT_COUNTER"
]


"""trace_select(data, "classname", "dpu-runner") -> [...]"""


def select_trace(trace_data, sel_by, val) -> []:
    return [d for d in trace_data if d.get(sel_by, None) == val]


"""trace_select(data, "classname", "dpu-runner") -> [...]"""


def select_trace_classes(trace_data, classes) -> []:
    if type(classes) != list:
        raise TypeError
    return [d for d in trace_data if d.get("classname", None) in classes]


class tracepointEvent:
    def __init__(self, l: dict):
        """event_state: 1: start, 0: end, -1: not defined"""
        self.event_state = int(l.pop("event_state", -1))
        self.pid = int(l.pop("pid", 0))
        self.cpu_id = int(l.pop("cpu_id", -1))
        self.t_class = l.pop("classname", "null")
        self.time_stamp = float(l.pop("ts", 0.0))
        self.payload = l

    def toTimelineEvent(self):
        if self.event_state == 1:
            et = "start"
        elif self.event_state == 0:
            et = "done"
        else:
            et = "marker"

        ct = self.t_class
        cid = self.payload.get("device_core_idx", -1)

        return vaiTimelineEvent(self.time_stamp, self.pid, et, ct, cid, self.payload)


def parse(l, options) -> tracepointEvent:
    """Selet trace clock"""
    tc = options.get('traceClock', None)

    return tracepointEvent(l.split(" ", 6))



# Copyright 2021 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functools import reduce
import sys
import os
import logging
import copy
from ctypes import *
import json
import logging
import binascii

from . import dpu_mc_parser


SUBGRAPH_DB = {}
DPU_NAME = ""


class subgraph_info:
    def __init__(self):
        pass


class xmodel_info:
    def __init__(self, name):
        self.name = name
        self.subgraphs = []


def subg_to_id(sub_g, key="workload"):
    if key == "workload":
        workload = 0
        name = sub_g.get("subgraph_name")
        workload = sub_g.get("workload")
        return "%s|%d" % (name, workload)
    if key == "depth":
        name = sub_g.get("subgraph_name")
        depth = sub_g.get("depth")
        return "%s|%d" % (name, depth)


def idx_to_name(idx):
    return idx.rsplit('|', 2)[0]


def get_subg_info(sub_g: dict, rank_id):
    """ Example subg data
    {
    "subgraph_name":"subgraph_pool5","id":"0-3","dpu_name":"","device":"","workload":100352,"op_num":1,"i_tensors_shape":[[1,7,7,2048]],"o_tensors_shape":[[1,1,1,2048]],
    "mc_code_sstr.str()":
        "a0021001ff070000ff87017f004c02
         20a0121001ff070000ff87017f0084
         ...
        }
    """
    global DPU_NAME
    global SUBGRAPH_DB

    """Only update DPU IP name for once"""
    if (DPU_NAME == ""):
        DPU_NAME = sub_g.get("dpu_name", "")

    idx_w = subg_to_id(sub_g, "workload")

    name = sub_g.get("subgraph_name").strip()
    workload = sub_g.get("workload", 0)
    depth = sub_g.get("depth", 0)
    op_num = sub_g.get("op_num", 0)
    device = sub_g.get("device", "unknow")

    mc = sub_g.get("mc_code_sstr.str()", "")
    mc_size = len(mc)

    if (mc_size < 50_000_000):
        if (mc_size > 5_000_000):
            logging.warning(
                "vaitrace is analyzing .xmodel, this model is large, please wait for a while ")
        mc_info = dpu_mc_parser.process_mc(DPU_NAME, mc)
    else:
        logging.error(
            "vaitrace is analyzing .xmodel, this model is too large, vaitrace can not handle it now ")
        mc_info = [0, 0, 0, 0]

    load_io_img_size = mc_info[0]
    load_io_para_size = mc_info[1]
    save_io_size = mc_info[2]

    i_tensor_shape = sub_g.get("i_tensors_shape", [])
    o_tensor_shape = sub_g.get("o_tensors_shape", [])

    subg_info = {idx_w:
                 {"name": name,
                     "device": device,
                     "workload": workload,
                     "op_num": op_num,
                     "load_io_img_size": load_io_img_size,
                     "load_io_para_size": load_io_para_size,
                     "save_io_size": save_io_size,
                     "i_tensor_shape": i_tensor_shape,
                     "o_tensor_shape": o_tensor_shape,
                     "rank_id": rank_id}}

    if idx_w not in SUBGRAPH_DB.keys():
        SUBGRAPH_DB.update(subg_info)
    else:
        logging.info("SubGraph index duplication: [%s]" % idx_w)


"""input: {'classname': 'subgraph_info', 'info_file': '/tmp/vaitrace_subgraph_info_15230', 'section': 'INFO'}"""


def xmodel_get_info(raw_trace: []):
    SUBGRAPH_DB.clear()
    raw_x_info = []

    if len(raw_trace) == 0:
        logging.error(raw_trace)
        logging.error("xmodel info raw info error")
        return {}
    try:
        for item in raw_trace:
            info_file = item.get('info_file')
            logging.debug("Open xmodel information file: %s" % info_file)
            subgs = open(info_file, "r+t").readlines()
            logging.debug("Subgraphs to process: %d" % len(subgs))
            for subg in subgs:
                raw_x_info.append(subg)
    except:
        logging.error("xmodel parser error")
        return {}

    logging.info("Processing xmodel information")
    rank_id = 0
    for subg_str in raw_x_info:
        try:
            subg = json.loads(subg_str)
        except:
            logging.warning("Subgraph information format error")

        get_subg_info(subg, rank_id)
        rank_id += 1

    return SUBGRAPH_DB




# Copyright 2019 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os

import tracer.tracerBase
#import tracer.xir
#import tracer.xrt
import tracer.hwInfo
import tracer.function
import tracer.sched
#import tracer.cuEdge
import tracer.xapm
import tracer.cmd
import tracer.vart
import tracer.pyfunc

from tracer.tracerBase import *



"""
Copyright 2021 Xilinx Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import pandas as pd
import numpy as np
import time
import os

current_dir = os.path.dirname(__file__)
filename = os.path.join(current_dir, "data", "car_rental_training_data.csv")

data = pd.read_csv(filename, sep=';')
data.head()

complain_data = data[['Customer_Service', 'Satisfaction']]

from sklearn.model_selection import train_test_split
from tensorflow.python import keras
from tensorflow.python.keras import Sequential
from tensorflow.python.keras import layers
from tensorflow.python.keras.preprocessing.sequence import pad_sequences
from tensorflow.python.keras.preprocessing.text import Tokenizer
import tensorflow as tf

import datetime

max_features = 500
hidden_size = 100

for idx, row in complain_data.iterrows():
  row[0] = row[0].replace('rt',' ')

tokenizer = Tokenizer(num_words=max_features, split=' ')
tokenizer.fit_on_texts(complain_data['Customer_Service'].values)
X = tokenizer.texts_to_sequences(complain_data['Customer_Service'].values)

maxlen = 50
X = pad_sequences(X, maxlen=maxlen)

Y = complain_data['Satisfaction'].values
X_train, X_test, Y_train, Y_test = train_test_split(X,Y, test_size = 0.33, random_state = 42)

embedding_vector_length = 32

model = Sequential()
model.add(layers.Embedding(max_features, embedding_vector_length, input_length=maxlen))
model.add(layers.Conv1D(filters=32, kernel_size=3, padding='same', activation='relu'))
model.add(layers.MaxPooling1D(pool_size=2))
model.add(layers.LSTM(100, recurrent_activation='sigmoid'))
model.add(layers.Dense(1, activation='sigmoid'))
model.compile(loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])
model.summary()

filename = os.path.join(current_dir, 'data', 'complain_model.h5')

is_training = False
if is_training:
  model.fit(X_train, Y_train, validation_data=(X_test, Y_test), epochs=20, batch_size=64)
  # Evaluate the model
  scores = model.evaluate(X_test, Y_test, verbose=0)
  print("Evaluation Accuracy: %.2f%%" % (scores[1]*100))
  model.save(filename, save_format='tf')
else:
  model.load_weights(filename)

from tensorflow.python.keras import backend as K
import datetime as dt
#t1=time.time()
# layers: [Embedding, Conv1D, MaxPooling1D, LSTM, Dense]
#print("x_test size:", X_test.shape)
lstm_upstream = K.function([model.layers[0].input], [model.layers[2].output])
lstm_input = lstm_upstream([X_test])[0]

batches = lstm_input.shape[0]
lstm_output = np.zeros((batches, 100))
lstm_tmp = np.zeros((1, 100))
lstm = K.function([model.layers[3].input], [model.layers[3].output])
t1=time.time()
for index, x in enumerate(lstm_input):
    lstm_input_batch1 = x.reshape(1,25,32)
    lstm_output[index] = lstm(lstm_input_batch1)[0]

#lstm = K.function([model.layers[3].input], [model.layers[3].output])
#lstm_start = dt.datetime.now()
#lstm_output = lstm([lstm_input])[0]
#lstm_finish = dt.datetime.now()
#print('lstm foward time(secs):', (lstm_finish - lstm_start).total_seconds())
#lstm_out = lstm_output_batch1.reshape((batches, 25, 100))
lstm_downstream = Sequential()
lstm_downstream.add(layers.Dense(1, activation='sigmoid'))
lstm_downstream.compile(loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])
lstm_downstream.build((1, 100))
lstm_downstream.layers[0].set_weights(model.get_layer('dense').get_weights())
score = lstm_downstream.evaluate(lstm_output, Y_test, verbose=0)
t2=time.time()
print('Accuracy:', score[1])
print('E2E Time:', t2-t1)
#print(tf.__version__)



#Copyright 2021 Xilinx Inc.
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#MIT License

#Copyright (c) 2016 Gabriel Stanovsky
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in all
#copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#SOFTWARE.

''' 
Usage:
   benchmark --gold=GOLD_OIE --out=OUTPUT_FILE (--stanford=STANFORD_OIE | --ollie=OLLIE_OIE |--reverb=REVERB_OIE | --clausie=CLAUSIE_OIE | --openiefour=OPENIEFOUR_OIE | --props=PROPS_OIE | --tabbed=TABBED)

Options:
  --gold=GOLD_OIE              The gold reference Open IE file (by default, it should be under ./oie_corpus/all.oie).
  --out-OUTPUT_FILE            The output file, into which the precision recall curve will be written.
  --clausie=CLAUSIE_OIE        Read ClausIE format from file CLAUSIE_OIE.
  --ollie=OLLIE_OIE            Read OLLIE format from file OLLIE_OIE.
  --openiefour=OPENIEFOUR_OIE  Read Open IE 4 format from file OPENIEFOUR_OIE.
  --props=PROPS_OIE            Read PropS format from file PROPS_OIE
  --reverb=REVERB_OIE          Read ReVerb format from file REVERB_OIE
  --stanford=STANFORD_OIE      Read Stanford format from file STANFORD_OIE
  --tabbed=TABBED              Read tabbed format from file TABBED
'''
import docopt
import string
import numpy as np
from sklearn.metrics import precision_recall_curve
from sklearn.metrics import auc, f1_score, precision_recall_fscore_support
import re
import logging
logging.basicConfig(level = logging.INFO)

from oie_readers.stanfordReader import StanfordReader
from oie_readers.ollieReader import OllieReader
from oie_readers.reVerbReader import ReVerbReader
from oie_readers.clausieReader import ClausieReader
from oie_readers.openieFourReader import OpenieFourReader
from oie_readers.propsReader import PropSReader
from oie_readers.tabReader import TabReader

from oie_readers.goldReader import GoldReader
from matcher import Matcher

class Benchmark:
    ''' Compare the gold OIE dataset against a predicted equivalent '''
    def __init__(self, gold_fn):
        ''' Load gold Open IE, this will serve to compare against using the compare function '''
        gr = GoldReader() 
        gr.read(gold_fn)
        self.gold = gr.oie

    def compare(self, predicted, matchingFunc, output_fn):
        ''' Compare gold against predicted using a specified matching function. 
            Outputs PR curve to output_fn '''
        
        y_true = []
        y_scores = []
        
        correctTotal = 0
        unmatchedCount = 0        
        predicted = Benchmark.normalizeDict(predicted)
        gold = Benchmark.normalizeDict(self.gold)
                
        for sent, goldExtractions in list(gold.items()):
            if sent not in predicted:
                # The extractor didn't find any extractions for this sentence
                unmatchedCount += len(goldExtractions)
                correctTotal += len(goldExtractions)
                continue
                
            predictedExtractions = predicted[sent]
            
            for goldEx in goldExtractions:
                correctTotal += 1
                found = False
                
                for predictedEx in predictedExtractions:
                    if matchingFunc(goldEx, 
                                    predictedEx, 
                                    ignoreStopwords = True, 
                                    ignoreCase = True):
                        
                        y_true.append(1)
                        y_scores.append(predictedEx.confidence)
                        predictedEx.matched.append(output_fn)

                        # Also mark any other predictions with the
                        # same exact predicate as matched.
                        # This is to support packages that do conjunction
                        # splitting, and doesn't affect the results for
                        # packages that don't.
                        if predictedEx.splits_conjunctions:
                            for otherPredictedEx in predictedExtractions:
                                if otherPredictedEx.pred == predictedEx.pred:
                                    otherPredictedEx.matched.append(output_fn)

                        found = True
                        break
                    
                if not found:
                    unmatchedCount += 1
                    
            for predictedEx in [x for x in predictedExtractions if (output_fn not in x.matched)]:
                # Add false positives
                y_true.append(0)
                y_scores.append(predictedEx.confidence)
                
        y_true = y_true
        y_scores = y_scores
        # recall on y_true, y  (r')_scores computes |covered by extractor| / |True in what's covered by extractor|
        # to get to true recall we do r' * (|True in what's covered by extractor| / |True in gold|) = |true in what's covered| / |true in gold|
        p, r, thr = Benchmark.prCurve(np.array(y_true), np.array(y_scores),
                       recallMultiplier = ((correctTotal - unmatchedCount)/float(correctTotal)))
        #f1_scores = 2*(p*r)/(p+r)
        # write PR to file
        with open(output_fn, 'w') as fout:
            fout.write('{0}\t{1}\n'.format("Precision", "Recall"))
            for cur_p, cur_r in sorted(zip(p, r), key = lambda cur_p_cur_r: cur_p_cur_r[1]):
                fout.write('{0}\t{1}\n'.format(cur_p, cur_r))
        #print('fake thr: ', thr[np.nanargmax(f1_scores)])
        #print('fake f1 : ', np.nanmax(f1_scores))
        print('auc: ', auc(r, p))
        thr_gold = 1.204235e-12
        for i, score in enumerate(y_scores):
            if score > thr_gold:
                y_scores[i] = 1.0
            else:
                y_scores[i] = 0.0
        #p, r, thr = Benchmark.prCurve(np.array(y_true), np.array(y_scores),
        #               recallMultiplier = ((correctTotal - unmatchedCount)/float(correctTotal)))
        recallMultiplier = (correctTotal - unmatchedCount)/float(correctTotal)
        p, r, _, _ = precision_recall_fscore_support(y_true, np.array(y_scores), average="binary")
        r *= recallMultiplier
        #print(p, r)
        f1_scores = 2*(p*r)/(p+r)
        print('f1 : ', f1_scores)
    @staticmethod
    def prCurve(y_true, y_scores, recallMultiplier):
        # Recall multiplier - accounts for the percentage examples unreached by 
        precision, recall, thr = precision_recall_curve(y_true, y_scores)
        recall = recall * recallMultiplier
        return precision, recall, thr

    # Helper functions:
    @staticmethod
    def normalizeDict(d):
        return dict([(Benchmark.normalizeKey(k), v) for k, v in list(d.items())])
    
    @staticmethod
    def normalizeKey(k):
        return Benchmark.removePunct(str(Benchmark.PTB_unescape(k.replace(' ',''))))

    @staticmethod
    def PTB_escape(s):
        for u, e in Benchmark.PTB_ESCAPES:
            s = s.replace(u, e)
        return s
    
    @staticmethod
    def PTB_unescape(s):
        for u, e in Benchmark.PTB_ESCAPES:
            s = s.replace(e, u)
        return s
    
    @staticmethod
    def removePunct(s):
        return Benchmark.regex.sub('', s)
    
    # CONSTANTS
    regex = re.compile('[%s]' % re.escape(string.punctuation))
    
    # Penn treebank bracket escapes 
    # Taken from: https://github.com/nlplab/brat/blob/master/server/src/gtbtokenize.py
    PTB_ESCAPES = [('(', '-LRB-'),
                   (')', '-RRB-'),
                   ('[', '-LSB-'),
                   (']', '-RSB-'),
                   ('{', '-LCB-'),
                   ('}', '-RCB-'),]


if __name__ == '__main__':
    args = docopt.docopt(__doc__)
    logging.debug(args)
    if args['--stanford']:
        predicted = StanfordReader()
        predicted.read(args['--stanford'])
    
    if args['--props']:
        predicted = PropSReader()
        predicted.read(args['--props'])
       
    if args['--ollie']:
        predicted = OllieReader()
        predicted.read(args['--ollie'])
    
    if args['--reverb']:
        predicted = ReVerbReader()
        predicted.read(args['--reverb'])
    
    if args['--clausie']:
        predicted = ClausieReader()
        predicted.read(args['--clausie'])
        
    if args['--openiefour']:
        predicted = OpenieFourReader()
        predicted.read(args['--openiefour'])
    if args['--tabbed']:
        predicted = TabReader()
        predicted.read(args['--tabbed'])


    b = Benchmark(args['--gold'])
    out_filename = args['--out']

    logging.info("Writing PR curve of {} to {}".format(predicted.name, out_filename))
    b.compare(predicted = predicted.oie, 
               matchingFunc = Matcher.lexicalMatch,
               output_fn = out_filename)
    
        
        



#!/usr/bin/python3
# -*- coding:utf-8 -*-

# Copyright 2019 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ctypes
import json
import tracer.tracerBase


class pyfuncTracer(tracer.tracerBase.Tracer):
    def __init__(self):
        super().__init__('pyfunc', source=['tracepoint'], compatible={
            'machine': ["x86_64", "aarch64"]})
        self.traceLog = []
        self.timesync = 0

    def process(self, data, t_range=[]):
        for s in self.source:
            d = data.get(s, None)
            if d is None:
                continue
            for l in d:
                try:
                    """EVENT_TIME_SYNC 0 -1 XRT -1 0.000000610 6285.087307830"""
                    # if l.startswith('EVENT_TIME_SYNC'):
                    #    xrt_t = float(l.strip().split()[5])
                    #    steady_t = float(l.strip().split()[6])
                    #    self.timesync = steady_t - xrt_t
                    # if l.find(PYFUNC_FILTER_TAGS) < 0:
                    if l.get("classname", None) != "py":
                        continue
                except:
                    continue

                self.traceLog.append(l)

    def start(self):
        super().start()

    def stop(self):
        libvart_trace = ctypes.CDLL('libvart-trace.so')
        f_trace_stop = libvart_trace.stop
        f_trace_stop.argtypes = None
        f_trace_stop.restype = None

        f_trace_stop()
        super().stop()

    def prepare(self, options: dict, debug: bool):
        self.saveTo = None
        if debug:
            self.saveTo = "./%s.trace" % self.name

    def getData(self):
        if self.saveTo != None:
            with open(self.saveTo, "w+t") as save:
                save.writelines(json.dumps(self.traceLog))
        return self.traceLog


tracer.tracerBase.register(pyfuncTracer())



#Copyright 2021 Xilinx Inc.
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.

#MIT License
#
#Copyright (c) 2016 Gabriel Stanovsky
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in all
#copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#SOFTWARE.

"""Usage:
    <file-name> --in=INPUT_FILE --out=OUTPUT_FILE
"""
from docopt import docopt

def run(inf, out):
    tab_output = open(out, 'w')

    with open(inf, 'r') as f:
        for line in f:
            items = line.strip().split("\t")
            sent = items[0]
            conf = items[-1]
            args = []
            for arg in items[1:-1]:
                if arg.startswith("V"):
                    pred = arg[arg.index(':') + 1:]
                elif arg.startswith("ARG"):
                    args.append(arg[arg.index(':') + 1:])
            newline = "{}\t{}\t{}\t{}\n".format(sent, conf, pred, "\t".join(args))
            tab_output.write(newline)

    tab_output.close()

if __name__=="__main__":
    args = docopt(__doc__)
    inf = args['--in']
    out = args['--out']
    run(inf, out)




# Copyright 2019 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tracer.tracerBase
import sys
import os
import json
import ctypes


class schedTracer(tracer.tracerBase.Tracer):
    def __init__(self):
        super().__init__('sched', source=["ftrace"], compatible={
            'machine': ["aarch64"]})

    def prepare(self, option: dict, debug: bool):
        #
        # dpu.enableEvent("sched", "sched_switch",\
        #     """(prev_comm == "%s") || (next_comm == "%s")""" % (comm, comm))
        #
        "Handle Input Options"
        #comm = args.cmd[0].split('/')[-1][:15]
        comm = option['control']['cmd'][0].split('/')[-1][:15]
        python_mode = option.get("cmdline_args", {}).get('python', False)
        if python_mode:
            comm = "vaitrace"

        "Handle Output Options"
        saveTo = None
        if debug:
            saveTo = './sched.trace'

        optForFtrace = {
            "collector": {
                "ftrace": {
                    "sched": {
                        "name": "sched",
                        "type": "event",
                        "saveTo": saveTo,
                        "traceList": [
                            ["sched", "sched_process_exec"],
                            ["sched", "sched_process_fork"],
                            ["sched", "sched_process_exit"],
                            ["sched", "sched_switch",
                             """(prev_comm == "%s") || (next_comm == "%s")""" % (comm, comm)]
                        ]
                    }
                }
            }
        }

        return optForFtrace

    def process(self, data, t_range=[]):
        self.data = [l for l in data.get('ftrace', {}).get(
            self.name) if not l.startswith('#')]

    def getData(self):
        return self.data


tracer.tracerBase.register(schedTracer())
